/***************************************************************************************
 * Project  :SC16IS752
 * Describe :UART0 communicates with UART1
 *			 GPIO0-GPIO3 levels flip every second
 *           read GPIO4-GPIO7 levels every second
 * Experimental Platform :Raspberry Pi 4B + Serial Extension Module
 * Hardware Connection : Header
 *	I2C_Address:	A0 -> 3V3		IRQ_PIN:	IRQ -> P5
 *			        A2 -> 3V3					
 * Library Version :WiringPi_V2.52
 * Author		   :Christian
 * Web Site		   :www.seengreat.com
***************************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h> 
#include <signal.h>
#include <unistd.h>
#include <wiringPi.h>
#include <wiringSerial.h>
#include "sc16is752.h"

int UART0_fd,UART1_fd;

void Handler(int signo)
{
    printf("Handler:Serial Close \r\n");
    printf("Handler:GPIO Stop\r\n");
    serialClose(UART0_fd);
    serialClose(UART1_fd);
    SC16IS752_GPIO_Disable();
    exit(0);
}
/*****************************************************
 * This demo codes sets GPIO0~3 as output,GPIO4~7 as
 * input,communicates with each other between ttySC0 and 
 * ttySC1,then inverts the output state of GPIO0~3 and 
 * read the input state of GPIO4~7 every second,
 * Here is the wiring definition:
 * TXDA-----RXDB
 * RXDA-----TXDB
 * **************************************************/
int main()
{
    int val=0;
    unsigned char num,len;
    unsigned char uart0_rev[255],uart0_send[]="Hello word";
    unsigned char uart1_rev[255],uart1_send[]="www.seengreat.com";
    unsigned char out_val = 0;
    
    if((UART0_fd = serialOpen ("/dev/ttySC0", 115200)) == -1)
		printf("init ttySC0 error\n");
    if((UART1_fd = serialOpen ("/dev/ttySC1", 115200)) == -1)
		printf("init ttySC1 error\n");
    
    SC16IS752_GPIO_Enable();
    for(num=0;num<4;num++)
        SC16IS752_GPIO_Mode(num, OUT);
    for(num=4;num<8;num++)
        SC16IS752_GPIO_Mode(num, IN);
    signal(SIGINT, Handler);//Exception handling:ctrl + c
    while(1)
    {
        printf("\r\nStart sending data\r\n");
        write(UART0_fd,uart0_send,sizeof(uart0_send));
        delay(10);
        write(UART1_fd,uart1_send,sizeof(uart1_send));
        delay(100);
        
        printf("UART0 received data is:");
        len=serialDataAvail(UART0_fd);
        read(UART0_fd,uart0_rev,len);
        printf("%s\r\n",uart0_rev); 
        memset(uart0_rev,'\0',sizeof(uart0_send));    
        serialFlush(UART0_fd);
        
        printf("UART1 received data is:");
        len=serialDataAvail(UART1_fd);
        read(UART1_fd,uart1_rev,len);
        printf("%s\r\n",uart1_rev);
        memset(uart1_rev,'\0',sizeof(uart1_send));
        serialFlush(UART1_fd);
        
        for(num=0;num<4;num++)
        {
            SC16IS752_GPIO_Write(num, out_val);
        }
        for(num=4;num<8;num++)
        {
            val = SC16IS752_GPIO_Read(num);
            printf("GPIO %d is %d\r\n",num, val);
        }
        if(out_val==0)
        {
            out_val = 1;    
        }
        else
        {
            out_val = 0;    
        }
        delay(1000);
    }
    SC16IS752_GPIO_Disable();
    return 0;
}

