/***************************************************************************************
 * Project  :SC16IS752
 * Describe :UART0 communicates with UART1
 *			 GPIO0-GPIO7 levels flip every second
 * Experimental Platform :Raspberry Pi 4B + Serial Extension Module
 * Hardware Connection : Header
 *	I2C_Address:	A0 -> 3V3		IRQ_PIN:	IRQ -> P5
 *			        A2 -> 3V3					
 * Library Version :WiringPi_V2.52
 * Author		   :Christian
 * Web Site		   :www.seengreat.com
***************************************************************************************/
#include <stdio.h>
#include <stdlib.h> 
#include <fcntl.h>
#include <unistd.h>
#include "sc16is752.h"

int SC16IS752_GPIO_Export(int Pin)
{
    char buffer[NUM_MAXBUF];
    int len,fd;

    fd = open("/sys/class/gpio/export", O_WRONLY);
    if (fd < 0) 
    {
        printf( "Export Failed: GPIO%d\n", Pin - PIN_START);
        return -1;
    }

    len = snprintf(buffer, NUM_MAXBUF, "%d", Pin);
    write(fd, buffer, len);
    
    printf( "Export: GPIO%d\r\n", Pin - PIN_START);

    close(fd);
    return 0;
}

int SC16IS752_GPIO_Unexport(int Pin)
{
    char buffer[NUM_MAXBUF];
    int len,fd;

    fd = open("/sys/class/gpio/unexport", O_WRONLY);
    if (fd < 0) 
    {
        printf( "unexport Failed: GPIO%d\n", Pin - PIN_START);
        return -1;
    }

    len = snprintf(buffer, NUM_MAXBUF, "%d", Pin);
    write(fd, buffer, len);
    
    printf( "Unexport: GPIO%d\r\n", Pin - PIN_START);
    
    close(fd);
    return 0;
}

int SC16IS752_GPIO_Direction(int Pin, int Dir)
{
    char path[DIR_MAXSIZ];
    int fd;
    
    snprintf(path, DIR_MAXSIZ, "/sys/class/gpio/gpio%d/direction", Pin);
    fd = open(path, O_WRONLY);
    if (fd < 0) 
    {
        printf( "Set Direction failed: Pin%d\n", Pin - PIN_START);
        return -1;
    }

    if(Dir == IN)
    {
        write(fd,"in",2);
        printf("GPIO%d:intput\r\n", Pin - PIN_START);
    }
    else
    {
        write(fd,"out",3);
        printf("GPIO%d:Output\r\n", Pin - PIN_START);
    }
    
    close(fd);
    return 0;
}

int SC16IS752_GPIO_Read(int Pin)
{
    char path[DIR_MAXSIZ],value_str[3];
    int fd;
    
    Pin = Pin + PIN_START;
    snprintf(path, DIR_MAXSIZ, "/sys/class/gpio/gpio%d/value", Pin);
    fd = open(path, O_RDONLY);
    if (fd < 0) 
    {
        printf( "Read failed GPIO%d\n", Pin - PIN_START);
        return -1;
    }

    if (read(fd, value_str, 3) < 0) 
    {
        printf( "failed to read value!\n");
        return -1;
    }

    close(fd);
    return(atoi(value_str));
}

int SC16IS752_GPIO_Write(int Pin, int value)
{
    char path[DIR_MAXSIZ];
    int fd;
    
    Pin = Pin + PIN_START;    
    snprintf(path, DIR_MAXSIZ, "/sys/class/gpio/gpio%d/value", Pin);
    fd = open(path, O_WRONLY);
    if (fd < 0) 
    {
        printf( "Write failed : GPIO%d,value = %d\n", Pin - PIN_START, value);
        return -1;
    }

    if(value == LOW)
        write(fd,"0",1);
    else
        write(fd,"1",1);
    
    close(fd);
    return 0;
}

void SC16IS752_GPIO_Mode(int Pin, int Mode)
{
    Pin += PIN_START;
    SC16IS752_GPIO_Direction(Pin, Mode);
}

void SC16IS752_GPIO_Enable(void)
{
    unsigned char num;
    for(num=0;num<8;num++)
        SC16IS752_GPIO_Export(num + PIN_START);
}

void SC16IS752_GPIO_Disable(void)
{
    unsigned char num;
    for(num=0;num<8;num++)
        SC16IS752_GPIO_Unexport(num + PIN_START);
}

